package cht.svista;
/******************************************************************************
 This material is confidential and proprietary to Boston University 
 and is protected under the laws of trade secret.  Distribution or use of 
 this material, or any portion thereof, other than as specifically authorized 
 by Boston University is expressly prohibited. If you have any questions about 
 the authorization of the code, please contact Dr. Zhenjun HU (zjhu@bu.edu)
 $version: 1.01 $
 $Date: 2003/10/01 $
 $Modification History: 2003/10/01 by Zhenjun Hu$
 $                      2004/10/01 by Zhenjun Hu$
 $Author: Zhenjun Hu $
*******************************************************************************/
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.URL;
import cht.svista.gv.ImageFactory;
import cht.svista.gv.GeneGraph;
import cht.svista.gv.GeneEditor;
import cht.svista.gv.GeneTree;
import cht.svista.gv.ControlPanel;
import cht.svista.gv.HelpAction;
import cht.svista.gv.AboutAction;
import cht.svista.gv.FeatureTableAction;
import cht.svista.pps.ComparisonAction;
import cht.svista.gv.ToolTipDelayAction;
import cht.svista.gv.TaskQueuePanel;

/**
The APIs listed below are designed for the purpose to let other software which wants to use SeqVISTA
as a visualization package for the bioseqs (protein/nucleotide sequence). It is not designed, however,
to allow other software to take the full control of the SeqVISTA (i.e, to modify all visual proterties
the feature components through programming). With this goal in mind, we only provide the API to get the instance
of SeqVISTA and set/add sequence records. 
<BR>
When using SeqVISTA for the inetgration purpose, There are two ways:
<UL>
	<LI>Light-weight integration. In this case, SeqVISTA is rather independed from the application being integrated. 
	and the method getInstance() should be called to get an instance (singleton) of SeqVISTA:
as shown below:<br><UL>
	SeqVISTA vista=SeqVISTA.getInstance();
</UL>
	<LI>Heavy-weight integration. In this case, SeqVISTA can be used as a Swing component and can be embedded
	in the application to be integrated. Multiple instances of SeqVISTA can be invoked through the construct SeqVISTA()
	as shown as following:<br><UL>
	SeqVISTA vista=new SeqVISTA();
	</UL>
</UL>
<br><H3>Three ways to load sequences</H3>
	<UL>
	<LI>Load sequences from a File object. The input should be a valida File object, SeqVISTA then transfer 
			the File object to an URL object. The other boolean input is used to tell SeqVISTA whether the existing
			sequences in SeqVISTA should be removed.
	<LI>Load sequences from URL. Enable SeqVISTA to load data from a given URL. If format is not specified, SeqVISTA
			will try to recognize it by itself through the registered Parsers.	
	<LI>Load sequences from a give text record. In the case, format must be specified unless it is GenBank format.
			In addition to <A HREF="http://www.ncbi.nlm.nih.gov">GenBank</A> HTML/flat file format, the supported formats 
			of the text are listed as following:
		<UL>
			<LI>gid: File of gi numbers. A file with GI numbers separated by tab, space or ","
			<LI>rem:<A HREF="http://www.repeatmasker.org/">Repeat Masker</A> output
			<LI>cis:ZLab <A HREF="http://zlab.bu.edu/~mfrith/cister.shtml">Cister</A> Output
			<LI>gbs:output of <A HREF="http://bayesweb.wadsworth.org/gibbs/gibbs.html">Gibbs Motif Sampler</A>
			<LI>ppd:<A HREF="http://bioinf.cs.ucl.ac.uk/psipred/">PsiPred(Protein 2nd Structure)</A> output
			<LI>mzef:<A HREF="http://rulai.cshl.org/tools/genefinder/">Michael Zhang Exon Finder</A> output
			<LI>fasta:FASTA File
			<LI>gsn:<A HREF="http://genes.mit.edu/GENSCAN.html">GenScan</A> output
			<LI>embl:<A HREF="http://www.embl.org/">EMBL</A> File
		</UL>
	</UL>
	
@version 2.01 10/01/2004
@author Zhenjun Hu
*/

public class SeqVISTA extends JPanel implements Runnable {
   private GeneGraph geneppp;
   private GeneEditor editor;
   private GeneTree treeView;
   private ControlPanel pControl;
	 private final JFrame mframe;
   private static boolean blIntegration=false;
   private static SeqVISTA vista=null;
   
   public void run() {if (mframe!=null) mframe.show();}
   public ControlPanel getControl(){return pControl;}
		
	 /**
		Use this construct to create a new instance of SeqVISTA that can be used as 
		a swing component to be embedded in other application. The SeqVISTA instance 
		invoked by this construct will have empty sequence record. Mutlple 
		instances are allowed. All instances will run in the integration mode.
		@see #getInstance()
		*/
   public SeqVISTA() {this(null, null);}
   private SeqVISTA(JFrame ff) {this(null, ff);}
   private SeqVISTA(JApplet mmm, JFrame ff){
		mframe=ff;
   	if (mmm!=null) ImageFactory.setApplet(mmm);
   	try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			//PopupFactory.setSharedInstance(new PopupFactory()); 
		} catch (Exception exc) {
			System.err.println("Error loading L&F: " + exc);
		}
		
 		
    setLayout(new BorderLayout());
    //cht.svista.gv.ResourceManager.loadConfiguration();
    
    geneppp=new GeneGraph(blIntegration || (mmm==null && ff==null));
    treeView=new GeneTree(geneppp.getPic().getAll());
    
    editor=new GeneEditor(geneppp.getPic());
    treeView.setEditor(editor);
    treeView.setPreferredSize(new Dimension(150, 550));
    geneppp.setEditor(editor);
    JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    JScrollPane sc=new JScrollPane(geneppp);
    sc.getVerticalScrollBar().setUnitIncrement(15);
    geneppp.getPic().setScroll(sc);
    splitPane.setTopComponent(sc);
    JScrollPane scedit=new JScrollPane(editor);
    scedit.getVerticalScrollBar().setUnitIncrement(30);
		splitPane.setBottomComponent(scedit);
		splitPane.setDividerLocation(220);
		splitPane.setDividerSize(2);
		
    JSplitPane overAllPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
   	overAllPane.setDividerLocation(150);
   	overAllPane.setDividerSize(2);
   	overAllPane.setPreferredSize(new Dimension(750, 580));
   	overAllPane.setRightComponent(splitPane);
   	overAllPane.setLeftComponent(treeView);
   	add(overAllPane, BorderLayout.CENTER);
   	pControl=new ControlPanel(geneppp, treeView, editor);
   	//ImageFactory.setControl(pControl);
   	geneppp.getPic().setControl(pControl);
   	treeView.setControl(pControl);
   	editor.setControl(pControl);
   	geneppp.getPic().init();
   	editor.init();
   	treeView.init();
   	cht.svista.gv.ResourceManager.addBookMark(pControl);
		if (ff!=null) add (pControl, BorderLayout.NORTH);
		if (!(blIntegration || (mmm==null && ff==null))) editor.displayGeneInfo(geneppp.getPic().getGene(), true);
		TaskQueuePanel tp=new TaskQueuePanel(overAllPane);
		pControl.setQueue(tp);
		add (tp, BorderLayout.SOUTH);
		JMenuBar mBar=makeMenu();
		if (ff!=null) ff.setJMenuBar(mBar);
		//following two lines are the workaround for the window displaying bug, see pControl for detail.
//		pControl.getLocationBox().setSelectedIndex(0);
//		pControl.getLocationBox().setEditable(false);
  }
  
  
	private JMenuBar makeMenu(){
		ActionListener pHandler =geneppp.getPic().getMenuHandler();
		JMenuBar mb = new JMenuBar();
		JMenu file=new JMenu("File");
		file.setMnemonic(KeyEvent.VK_F);
		file.add(new ComparisonAction(geneppp.getPic(), -1));
		JMenuItem[] fItems={
			new JMenuItem("Open as ...", ImageFactory.loadImageIcon(this,"images/open.gif","Save the Image")),
			new JMenuItem("Save as SeqVISTA file", ImageFactory.loadImageIcon(this,"images/export.gif","Save the Image")),
			new JMenuItem("Save the Image", ImageFactory.loadImageIcon(this,"images/save.gif","Save the Image"))
		};
		fItems[0].setActionCommand("openseq");
		fItems[1].setActionCommand("saveseq");
		fItems[2].setActionCommand("saveimage");
		for (int i=0;i<fItems.length;i++){
			fItems[i].addActionListener(pHandler);
			file.add(fItems[i]);
		} 
		file.addSeparator();
		JMenuItem exit=new JMenuItem("Exit", ImageFactory.loadImageIcon(this,"images/close.gif","Exit"));
		exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.ALT_MASK));
		exit.addActionListener(new ActionListener(){
			public void actionPerformed( ActionEvent e ){cht.svista.gv.ResourceManager.saveBookMark(pControl);System.exit(0);}});
		JMenuItem hidden=new JMenuItem("Save Sequence");
		hidden.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, Event.CTRL_MASK ));
		hidden.setVisible(false);
		hidden.setActionCommand("hiddensave");
		hidden.addActionListener(pHandler);
		file.add(exit);
		file.add(hidden);
		JMenu edit=new JMenu("Edit");
		edit.setMnemonic(KeyEvent.VK_E);
		edit.add(new cht.svista.pps.FeaturePropAction("Add New Feature", 0, pControl));
		edit.add(new cht.svista.pps.FeaturePropAction("Edit the Feature", 1, pControl));
		edit.add(new cht.svista.pps.FeaturePropAction("Hide the Feature", 2, pControl));
		edit.add(new cht.svista.pps.FeaturePropAction("Remove the Feature", 3, pControl));
		edit.addSeparator();
		edit.add(treeView.getTreeMenu());
		edit.add(geneppp.getPic().getPicEditMenu());
		edit.add(editor.getEditorMenu());
		
		JMenu visu=new JMenu("Visualization");
		visu.setMnemonic(KeyEvent.VK_V);
		JMenuItem[] vItems={
			new JMenuItem("Reset", ImageFactory.loadImageIcon(this,"images/reset.gif","Reset")),
			new JMenuItem("Zoom In", ImageFactory.loadImageIcon(this,"images/zoomin.gif","Zoom In")),
		};
		vItems[0].setActionCommand("reset");
		vItems[1].setActionCommand("zoomin");
		for (int i=0;i<vItems.length;i++){
			vItems[i].addActionListener(pHandler);
			visu.add(vItems[i]);
		}
		visu.addSeparator();
		visu.add(new ToolTipDelayAction());
		JCheckBoxMenuItem sf=new JCheckBoxMenuItem("Show Sub-Features", true);
		sf.setActionCommand("subfeature");
		sf.addActionListener(pHandler);
		visu.add(sf);
		visu.add(new FeatureTableAction(geneppp.getPic()));
		JMenu ana=new JMenu("Analysis");
		ana.setMnemonic(KeyEvent.VK_A);
		ana.add(geneppp.getPic().getDNASubMenu());
		ana.add(geneppp.getPic().getProSubMenu());
		JMenu com=new JMenu("Comparison");
		com.setMnemonic(KeyEvent.VK_C);
		JRadioButtonMenuItem par=new JRadioButtonMenuItem ("Parallel");
		par.setActionCommand("parallel");
		par.addActionListener(pHandler);
		par.setSelected(true);
		JRadioButtonMenuItem comp=new JRadioButtonMenuItem ("Composite");
		comp.setActionCommand("compositive");
		comp.addActionListener(pHandler);
		JRadioButtonMenuItem scomp=new JRadioButtonMenuItem ("Composite with Selection");
		scomp.setActionCommand("s-compos");
		scomp.addActionListener(pHandler);
		ButtonGroup group = new ButtonGroup();
		group.add(par);group.add(comp);//group.add(scomp);
		com.add(par);com.add(comp);//com.add(scomp);
		JMenu adv=new JMenu("Advanced");
		adv.setMnemonic(KeyEvent.VK_A);
		JMenuItem[] aItems={
			new JMenuItem("Proxy Configuration", ImageFactory.loadImageIcon(this,"images/config.gif","Proxy Configuration")),
			new JMenuItem("Pipelined Task Manager", ImageFactory.getIcon(this, "task.gif"))
		};
		aItems[0].setActionCommand("proxy_config");//repeater finder
		aItems[1].setActionCommand("pipelined");
		for (int i=0;i<aItems.length;i++){
			aItems[i].addActionListener(pHandler);
			adv.add(aItems[i]);
		}
		
		JMenu help=new JMenu("Help");
		help.setMnemonic(KeyEvent.VK_H);
		help.add(new HelpAction());
		help.add(new AboutAction());
		 
	 	mb.add(file);
	 	mb.add(edit);
	 	mb.add(visu);
	 	mb.add(ana);
	 	mb.add(geneppp.getPic().getMotifMenu());
	 	mb.add(com);
	 	mb.add(adv);
	 	mb.add(help);
	 	return mb;
	}
   
	private static boolean isVersionOk(){
		double ver=cht.svista.gv.ResourceManager.getJDKVersion();
		if (ver<1.4){
			cht.svista.gv.InfoReport.errorInfo("SeqVISTA requires Java 1.4 or above to run.\nYour Java version is:"+System.getProperty("java.version", "1.0")+". \nSeqVISTA will be terminated.");
			return false;
		}
		return true;
	}
	public static void main( String args[] ){
		if (!isVersionOk()) System.exit(0);
		
		if (args.length>0) cht.svista.gv.ResourceManager.runType=args[0];
		//cht.svista.gv.ResourceManager.runType="1";
		//cht.svista.gv.InfoReport.info(cht.svista.gv.ResourceManager.runType);
		loadFrame(); 
		if (args.length>1) {
			//2nd parameter is the URL to be launched
			String location=args[0];
			location=location.replace('[','&');
			location=location.replace(']','=');
			//System.out.println(location);
			//System.out.println(args.length);
			//cht.svista.gv.InfoReport.info("args[0]");
			vista.getControl().setURL(location);
		}//else getControl().setURL("alad.gev");
	}
  private static void loadFrame(){ //loadFrame(null);}
	//public static void loadFrame(SeqUnitContainer suc){
   	
   	JWindow starter=new JWindow();
   	Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
   	if (!blIntegration){
   		JLabel jl=new JLabel (ImageFactory.loadImageIcon(cht.svista.gv.ResourceManager.fp,"images/start.gif", "SeqVISTA"));
   		//jl.setBorder(BorderFactory.createRaisedBevelBorder());
   		starter.getContentPane().add(jl, BorderLayout.CENTER);
   		starter.setLocation(d.width/2 - jl.getPreferredSize().width/2, d.height/2 - jl.getPreferredSize().height/2);
   		starter.pack();
   		starter.show();
		}
    final JFrame frame = new JFrame("SeqVISTA");
    SeqVISTA app = new SeqVISTA(frame);
    vista=app;
    ImageFactory.setFrame(frame);
    frame.setIconImage(ImageFactory.loadImageIcon(app,"images/gv.gif", "SeqVISTA").getImage());
    frame.getContentPane().add (app, BorderLayout.CENTER);
    int WIDTH = 750, HEIGHT = 650;
    frame.setSize(WIDTH, HEIGHT);
    
    frame.setLocation(d.width/2 - WIDTH/2, d.height/2 - HEIGHT/2);
    //frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    
    frame.addWindowListener(new WindowAdapter() {
    	public void windowClosing(WindowEvent e) {
    		cht.svista.gv.ResourceManager.saveBookMark(vista.getControl());
    		if (!blIntegration){
    			System.exit(0);
    		}else {
    			JFrame mff=ImageFactory.getFrame();
					if (mff!=null) mff.hide();
    		}
    	}
      public void windowDeiconified(WindowEvent e) {;}
      public void windowIconified(WindowEvent e) {;}
    });
    //JOptionPane.setRootFrame(frame);
    if (!blIntegration) starter.dispose();
    frame.pack();
	 	EventQueue.invokeLater(app);
	}
	/**
			This method should always be called when a reference of SeqVISTA is needed. It
			returns the reference of SeqVISTA singleton.
			SeqVISTA will be running in the integration mode when this method is called.
			If SeqVISTA is not visible, this method will make it visible.
			@see #getInstance (String strURL)
			@see #setVistaVisible(boolean blVisible)
	*/
	public static SeqVISTA getInstance (){ return getInstance(null);}
	/**
			This method starts SeqVISTA singleton and load the sequence from corresponding URL.
			Typical URL includes HTML url or file URL. This method should not be used
			if the format of sequecnes represented by the URL can not be recognized 
			by SeqVISTA (SeqVISTA used registered file extension to parse the file). 
			If there is no extension in the URL, this method can only recognize GenBank HTML or 
			flat file format.
			@see #loadFromURL(URL seqURL, boolean blAdd)
			@see #loadFromURL(URL seqURL, boolean blAdd, String format)
			@see #loadFromURL(URL seqURL, boolean blAdd, int seqType)
			@see #loadFromURL(URL seqURL, boolean blAdd, String format, int seqType)
			@see #setVistaVisible(boolean blVisible)
	*/
	public static SeqVISTA getInstance (String strURL){
		if (vista==null){
			blIntegration=true;
			cht.svista.gv.ResourceManager.runType="1";	//set as web starter, need to check with EBI to see how it will work out in anyway
			if (!isVersionOk()) return null;
			loadFrame(); 
			if (strURL!=null) vista.getControl().setURL(strURL);
		}else vista.setVistaVisible(true);
		return vista;
	}
	/**
		Show/hide SeqVISTA window
		@param blVisible true: set SeqVISTA visible;fase, hide SeqVISTA
	*/
	public void setVistaVisible(boolean blVisible){
		JFrame mff=ImageFactory.getFrame();
		if (mff!=null) mff.setVisible(blVisible);
	}
	
	/**
		remove SeqVISTA singleton, please also set your SeqVISTA reference to null
	*/
	public void remove(){
		JFrame mff=ImageFactory.getFrame();
		if (mff!=null) {
			mff.setVisible(false);
			mff.dispose();
		}
		vista=null;
	}
	/**
		Load sequences from a file. SeqVISTA will determine whether it is protein or nucleotide 
		sequence.
		@param fileName File to be loaded. 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromFile(File fileName, boolean blAdd){loadFromFile(fileName, blAdd,null, 0);}
	/**
		Load sequences from a file. Sequence type is specified by seqType.
		@param fileName File to be loaded. 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@param seqType 0:nucleotide sequence, 1: protein sequence 
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromFile(File fileName, boolean blAdd, int seqType){loadFromFile(fileName, blAdd,null, seqType);}
	/**
		Load sequences from a file. File format is specified
		@param fileName File to be loaded. 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@param format the string to represent the file format, if not specified by the extension of the file
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromFile(File fileName, boolean blAdd, String format){loadFromFile(fileName, blAdd,format, 0);}
	/**
		Load sequences from a file. File format is specified
		@param fileName File to be loaded. 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@param format the string to represent the file format, if not specified by the extension of the file.
		@param seqType 0:nucleotide sequence, 1: protein sequence 
		if format is null, SeqVISTA will use GenBank HTML or flat file format to interpret the file. 
		Other formats supported by SeqVISTA:<BR>
		<UL>
			<LI>gid: File of gi numbers. A file with GI numbers separated by tab, space or ","
			<LI>rem:<A HREF="http://www.repeatmasker.org/">Repeat Masker</A> output
			<LI>cis:ZLab <A HREF="http://zlab.bu.edu/~mfrith/cister.shtml">Cister</A> Output
			<LI>gbs:output of <A HREF="http://bayesweb.wadsworth.org/gibbs/gibbs.html">Gibbs Motif Sampler</A>
			<LI>ppd:<A HREF="http://bioinf.cs.ucl.ac.uk/psipred/">PsiPred(Protein 2nd Structure)</A> output
			<LI>mzef:<A HREF="http://rulai.cshl.org/tools/genefinder/">Michael Zhang Exon Finder</A> output
			<LI>fasta:FASTA File
			<LI>gsn:<A HREF="http://genes.mit.edu/GENSCAN.html">GenScan</A> output
			<LI>embl:<A HREF="http://www.embl.org/">EMBL</A> File
		</UL>
	*/
	public void loadFromFile(File fileName, boolean blAdd, String format, int seqType){
		try{
			URL furl=fileName.toURL();
			loadFromURL(furl, blAdd,format, seqType);
		}catch(Exception ex){
			cht.svista.gv.InfoReport.info(ex.toString());
		}
	}
	/**
		Load sequences from a URL (either HTTP URL, or file URL)
		@param seqURL URL of the sequences 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@see #loadFromURL(URL seqURL, boolean blAdd, String format, int seqType)
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromURL(URL seqURL, boolean blAdd){loadFromURL(seqURL, blAdd,null, 0);}
	/**
		Load sequences from a URL (either HTTP URL, or file URL), format is specified
		@param seqURL URL of the sequences 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@param format the string to represent the file format, if not specified by the url (ends with the .+recognizable format, such .embl). 
		@see #loadFromURL(URL seqURL, boolean blAdd, String format, int seqType)
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromURL(URL seqURL, boolean blAdd, String format){loadFromURL(seqURL, blAdd,format, 0);}
	/**
		Load sequences from a URL (either HTTP URL, or file URL), format is specified
		@param seqURL URL of the sequences 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@param seqType 0:nucleotide sequence, 1: protein sequence 
		@see #loadFromURL(URL seqURL, boolean blAdd, String format, int seqType)
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromURL(URL seqURL, boolean blAdd, int seqType){loadFromURL(seqURL, blAdd,null, seqType);}
	/**
		Load sequences from a URL (either HTTP URL, or file URL), format and sequence type are both specified
		@param seqURL URL of the sequences 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@param seqType 0:nucleotide sequence, 1: protein sequence 
		@param format the string to represent the file format, if not specified by the url (ends with the .+recognizable format, such .embl). 
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromURL(URL seqURL, boolean blAdd, String format, int seqType){
		//System.out.println(blAdd);
		geneppp.getPic().setEntryType(seqType);
		geneppp.getPic().setSeqAdd(blAdd);
		cht.svista.pps.GevReader reader=new cht.svista.pps.GevReader(geneppp, treeView,editor);
		if (format==null) reader.loadGevFile(seqURL);
			else reader.loadGevFile(seqURL, format);
		pControl.addToList(1, seqURL.toString());
	}
	
	/**
		Load sequences from a String. 
		@param sourceText The String of corresponding sequence record 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromText(String sourceText, boolean blAdd){loadFromText(sourceText, blAdd, null, 0);}
	/**
		Load sequences from a String. Format is specified
		@param sourceText The String of corresponding sequence record 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@param format the string to represent the file format, if not specified by the url (ends with the .+recognizable format, such .embl). 
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromText(String sourceText, boolean blAdd, String format){loadFromText(sourceText, blAdd,format, 0);}
	/**
		Load sequences from a String. Format and sequence type are both specified
		@param sourceText The String of corresponding sequence record 
		@param blAdd if false, remove all existing sequence before adding the new ones 
		@param format the string to represent the file format, if not specified by the url (ends with the .+recognizable format, such .embl). 
		@param seqType 0:nucleotide sequence, 1: protein sequence 
		@see #loadFromFile(File fileName, boolean blAdd, String format)
	*/
	public void loadFromText(String sourceText, boolean blAdd, String format, int seqType){//0, auto, 1 nuc, 2 protein
		//reader.loadGevFile(fileName);
//		try{
//			Thread.sleep(3000);
//		}catch(Exception ex){;}
		if (sourceText.trim().length()<1) {
			cht.svista.gv.InfoReport.info("Null text, please check your data");
			return;
		}
		geneppp.getPic().setEntryType(seqType);
		geneppp.getPic().setSeqAdd(blAdd);
		cht.svista.pps.GevReader reader=new cht.svista.pps.GevReader(geneppp, treeView,editor);
		reader.parsing(null, sourceText, format);
	}
	
	/**
		Clear all previous sequences shown in SeqVISTA
	*/
	public void clear(){
		geneppp.clear();
   	editor.clear();
   	treeView.reset(null);
		
	}
		
}

